@@ -19,9 +19,11 @@
 #include "cmd.hh"
 #include "diff_output.hh"
 #include "file_io.hh"
+#include "parallel_iter.hh"
 #include "restrictions.hh"
 #include "revision.hh"
 #include "rev_height.hh"
+#include "rev_output.hh"
 #include "simplestring_xform.hh"
 #include "transforms.hh"
 #include "app_state.hh"
@@ -41,319 +43,192 @@ using std::string;
 using std::vector;
 using std::priority_queue;
 
-using boost::lexical_cast;
-
-// The changes_summary structure holds a list all of files and directories
-// affected in a revision, and is useful in the 'log' command to print this
-// information easily.  It has to be constructed from all cset objects
-// that belong to a revision.
-
-struct
-changes_summary
-{
-  cset cs;
-  changes_summary(void);
-  void add_change_set(cset const & cs);
-  void print(ostream & os, size_t max_cols) const;
-};
-
-changes_summary::changes_summary(void)
-{
-}
-
-void
-changes_summary::add_change_set(cset const & c)
-{
-  if (c.empty())
-    return;
-
-  // FIXME: not sure whether it matters for an informal summary
-  // object like this, but the pre-state names in deletes and renames
-  // are not really sensible to union; they refer to different trees
-  // so mixing them up in a single set is potentially ambiguous.
-
-  copy(c.nodes_deleted.begin(), c.nodes_deleted.end(),
-       inserter(cs.nodes_deleted, cs.nodes_deleted.begin()));
-
-  copy(c.files_added.begin(), c.files_added.end(),
-       inserter(cs.files_added, cs.files_added.begin()));
-
-  copy(c.dirs_added.begin(), c.dirs_added.end(),
-       inserter(cs.dirs_added, cs.dirs_added.begin()));
-
-  copy(c.nodes_renamed.begin(), c.nodes_renamed.end(),
-       inserter(cs.nodes_renamed, cs.nodes_renamed.begin()));
-
-  copy(c.deltas_applied.begin(), c.deltas_applied.end(),
-       inserter(cs.deltas_applied, cs.deltas_applied.begin()));
-
-  copy(c.attrs_cleared.begin(), c.attrs_cleared.end(),
-       inserter(cs.attrs_cleared, cs.attrs_cleared.begin()));
-
-  copy(c.attrs_set.begin(), c.attrs_set.end(),
-       inserter(cs.attrs_set, cs.attrs_set.begin()));
-}
-
 static void
-print_indented_set(ostream & os,
-                   set<file_path> const & s,
-                   size_t max_cols)
+get_data(database & db,
+         file_path const & path, file_id const & id,
+         bool const from_db, data & unpacked)
 {
-  size_t cols = 8;
-  os << "       ";
-  for (set<file_path>::const_iterator i = s.begin();
-       i != s.end(); i++)
+  if (from_db)
     {
-      string str = lexical_cast<string>(*i);
-      if (str.empty())
-        str = "."; // project root
-      if (cols > 8 && cols + str.size() + 1 >= max_cols)
-        {
-          cols = 8;
-          os << "\n       ";
-        }
-      os << ' ' << str;
-      cols += str.size() + 1;
+      file_data dat;
+      db.get_file_version(id, dat);
+      unpacked = dat.inner();
     }
-  os << '\n';
-}
-
-void
-changes_summary::print(ostream & os, size_t max_cols) const
-{
-
-  if (! cs.nodes_deleted.empty())
+  else
     {
-      os << _("Deleted entries:") << '\n';
-      print_indented_set(os, cs.nodes_deleted, max_cols);
-    }
-
-  if (! cs.nodes_renamed.empty())
-    {
-      os << _("Renamed entries:") << '\n';
-      for (map<file_path, file_path>::const_iterator
-           i = cs.nodes_renamed.begin();
-           i != cs.nodes_renamed.end(); i++)
-        os << "        " << i->first
-           << " to " << i->second << '\n';
-    }
-
-  if (! cs.files_added.empty())
-    {
-      set<file_path> tmp;
-      for (map<file_path, file_id>::const_iterator
-             i = cs.files_added.begin();
-           i != cs.files_added.end(); ++i)
-        tmp.insert(i->first);
-      os << _("Added files:") << '\n';
-      print_indented_set(os, tmp, max_cols);
-    }
-
-  if (! cs.dirs_added.empty())
-    {
-      os << _("Added directories:") << '\n';
-      print_indented_set(os, cs.dirs_added, max_cols);
-    }
-
-  if (! cs.deltas_applied.empty())
-    {
-      set<file_path> tmp;
-      for (map<file_path, pair<file_id, file_id> >::const_iterator
-             i = cs.deltas_applied.begin();
-           i != cs.deltas_applied.end(); ++i)
-        tmp.insert(i->first);
-      os << _("Modified files:") << '\n';
-      print_indented_set(os, tmp, max_cols);
-    }
-
-  if (! cs.attrs_set.empty() || ! cs.attrs_cleared.empty())
-    {
-      set<file_path> tmp;
-      for (set<pair<file_path, attr_key> >::const_iterator
-             i = cs.attrs_cleared.begin();
-           i != cs.attrs_cleared.end(); ++i)
-        tmp.insert(i->first);
-
-      for (map<pair<file_path, attr_key>, attr_value>::const_iterator
-             i = cs.attrs_set.begin();
-           i != cs.attrs_set.end(); ++i)
-        tmp.insert(i->first.first);
-
-      os << _("Modified attrs:") << '\n';
-      print_indented_set(os, tmp, max_cols);
+      read_data(path, unpacked);
     }
 }
 
 static void
-do_external_diff(options & opts, lua_hooks & lua, database & db,
-                 cset const & cs, bool new_is_archived)
+dump_diff(lua_hooks & lua,
+          file_path const & left_path, file_path const & right_path,
+          file_id const left_id, file_id const right_id,
+          data const & left_data, data const & right_data,
+          diff_type const diff_format,
+          bool external_diff_args_given,
+          string external_diff_args,
+          string const & encloser,
+          ostream & output)
 {
-  for (map<file_path, pair<file_id, file_id> >::const_iterator
-         i = cs.deltas_applied.begin();
-       i != cs.deltas_applied.end(); ++i)
+  if (diff_format == external_diff)
     {
-      data data_old;
-      data data_new;
-
-      file_data f_old;
-      db.get_file_version(delta_entry_src(i), f_old);
-      data_old = f_old.inner();
-
-      if (new_is_archived)
-        {
-          file_data f_new;
-          db.get_file_version(delta_entry_dst(i), f_new);
-          data_new = f_new.inner();
-        }
-      else
-        {
-          read_data(delta_entry_path(i), data_new);
-        }
-
       bool is_binary = false;
-      if (guess_binary(data_old()) ||
-          guess_binary(data_new()))
+      if (guess_binary(left_data()) || guess_binary(right_data()))
         is_binary = true;
 
-      lua.hook_external_diff(delta_entry_path(i),
-                             data_old,
-                             data_new,
+      file_path path = right_path;
+      if (path.empty()) // use the left path for deletes
+        path = left_path;
+
+      lua.hook_external_diff(path,
+                             left_data,
+                             right_data,
                              is_binary,
-                             opts.external_diff_args_given,
-                             opts.external_diff_args,
-                             encode_hexenc(delta_entry_src(i).inner()(),
-                                           delta_entry_src(i).inner().made_from),
-                             encode_hexenc(delta_entry_dst(i).inner()(),
-                                           delta_entry_dst(i).inner().made_from));
+                             external_diff_args_given,
+                             external_diff_args,
+                             encode_hexenc(left_id.inner()(),
+                                           left_id.inner().made_from),
+                             encode_hexenc(right_id.inner()(),
+                                           right_id.inner().made_from));
     }
+  else
+    {
+      // 60 is somewhat arbitrary, but less than 80
+      string patch_sep = string(60, '=');
+      output << patch_sep << '\n';
+
+      // see the big comment in diff_output.cc about what paths should be
+      string left = left_path.as_internal();
+      if (left.empty())
+        left = "/dev/null";
+
+      string right = right_path.as_internal();
+      if (right.empty())
+        right = "/dev/null";
+
+      make_diff(left, right,
+                left_id, right_id,
+                left_data, right_data,
+                output, diff_format, encloser);
+    }
+
 }
 
 static void
 dump_diffs(lua_hooks & lua,
            database & db,
-           cset const & cs,
-           set<file_path> const & paths,
+           roster_t const & left_roster,
+           roster_t const & right_roster,
            std::ostream & output,
            diff_type diff_format,
-           bool new_is_archived,
-           bool old_is_archived,
-           bool show_encloser,
-           bool limit_paths)
-{
-  // 60 is somewhat arbitrary, but less than 80
-  string patch_sep = string(60, '=');
-
-  for (map<file_path, file_id>::const_iterator
-         i = cs.files_added.begin();
-       i != cs.files_added.end(); ++i)
-    {
-      if (limit_paths && paths.find(i->first) == paths.end())
-        continue;
-
-      output << patch_sep << '\n';
-      data unpacked;
-      vector<string> lines;
-
-      if (new_is_archived)
-        {
-          file_data dat;
-          db.get_file_version(i->second, dat);
-          unpacked = dat.inner();
-        }
-      else
-        {
-          read_data(i->first, unpacked);
-        }
-
-      std::string pattern("");
-      if (show_encloser)
-        lua.hook_get_encloser_pattern(i->first, pattern);
-
-      make_diff(i->first.as_internal(),
-                i->first.as_internal(),
-                i->second,
-                i->second,
-                data(), unpacked,
-                output, diff_format, pattern);
-    }
-
-  map<file_path, file_path> reverse_rename_map;
-
-  for (map<file_path, file_path>::const_iterator
-         i = cs.nodes_renamed.begin();
-       i != cs.nodes_renamed.end(); ++i)
-    {
-      reverse_rename_map.insert(make_pair(i->second, i->first));
-    }
-
-  for (map<file_path, pair<file_id, file_id> >::const_iterator
-         i = cs.deltas_applied.begin();
-       i != cs.deltas_applied.end(); ++i)
-    {
-      if (limit_paths && paths.find(i->first) == paths.end())
-        continue;
-
-      file_data f_old;
-      data data_old, data_new;
-
-      output << patch_sep << '\n';
-
-      if (old_is_archived)
-        {
-          db.get_file_version(delta_entry_src(i), f_old);
-          data_old = f_old.inner();
-        }
-      else
-        {
-          I(new_is_archived);
-          read_data(delta_entry_path(i), data_old);
-        }
-
-      if (new_is_archived)
-        {
-          file_data f_new;
-          db.get_file_version(delta_entry_dst(i), f_new);
-          data_new = f_new.inner();
-        }
-      else
-        {
-          I(old_is_archived);
-          read_data(delta_entry_path(i), data_new);
-        }
-
-      file_path dst_path = delta_entry_path(i);
-      file_path src_path = dst_path;
-      map<file_path, file_path>::const_iterator re;
-      re = reverse_rename_map.find(dst_path);
-      if (re != reverse_rename_map.end())
-        src_path = re->second;
-
-      std::string pattern("");
-      if (show_encloser)
-        lua.hook_get_encloser_pattern(src_path, pattern);
-
-      make_diff(src_path.as_internal(),
-                dst_path.as_internal(),
-                delta_entry_src(i),
-                delta_entry_dst(i),
-                data_old, data_new,
-                output, diff_format, pattern);
-    }
-}
-
-static void
-dump_diffs(lua_hooks & lua,
-           database & db,
-           cset const & cs,
-           std::ostream & output,
-           diff_type diff_format,
-           bool new_is_archived,
-           bool old_is_archived,
+           bool external_diff_args_given,
+           string external_diff_args,
+           bool left_from_db,
+           bool right_from_db,
            bool show_encloser)
 {
-  set<file_path> dummy;
-  dump_diffs(lua, db, cs, dummy, output,
-             diff_format, new_is_archived, old_is_archived, show_encloser, false);
+  parallel::iter<node_map> i(left_roster.all_nodes(), right_roster.all_nodes());
+  while (i.next())
+    {
+      MM(i);
+      switch (i.state())
+        {
+        case parallel::invalid:
+          I(false);
+
+        case parallel::in_left:
+          // deleted
+          if (is_file_t(i.left_data()))
+            {
+              file_path left_path, right_path;
+              left_roster.get_name(i.left_key(), left_path);
+              // right_path is null
+          
+              file_id left_id, right_id;
+              left_id = downcast_to_file_t(i.left_data())->content;
+              // right_id is null
+
+              data left_data, right_data;
+              get_data(db, left_path, left_id, left_from_db, left_data);
+              // right_data is null
+
+              string encloser("");
+              if (show_encloser)
+                lua.hook_get_encloser_pattern(left_path, encloser);
+
+              dump_diff(lua, 
+                        left_path, right_path,
+                        left_id, right_id,
+                        left_data, right_data,
+                        diff_format, external_diff_args_given, external_diff_args,
+                        encloser, output);
+            }
+          break;
+
+        case parallel::in_right:
+          // added
+          if (is_file_t(i.right_data()))
+            {
+              file_path left_path, right_path;
+              // left_path is null
+              right_roster.get_name(i.right_key(), right_path);
+           
+              file_id left_id, right_id;
+              // left_id is null
+              right_id = downcast_to_file_t(i.right_data())->content;
+
+              data left_data, right_data;
+              // left_data is null
+              get_data(db, right_path, right_id, right_from_db, right_data);
+
+              string encloser("");
+              if (show_encloser)
+                lua.hook_get_encloser_pattern(right_path, encloser);
+
+              dump_diff(lua, 
+                        left_path, right_path,
+                        left_id, right_id,
+                        left_data, right_data,
+                        diff_format, external_diff_args_given, external_diff_args,
+                        encloser, output);
+            }
+          break;
+
+        case parallel::in_both:
+          // moved/renamed/patched/attribute changes
+          if (is_file_t(i.left_data()))
+            {
+          
+              file_id left_id, right_id;
+              left_id = downcast_to_file_t(i.left_data())->content;
+              right_id = downcast_to_file_t(i.right_data())->content;
+
+              if (left_id == right_id)
+                continue;
+
+              file_path left_path, right_path;
+              left_roster.get_name(i.left_key(), left_path);
+              right_roster.get_name(i.right_key(), right_path);
+          
+              data left_data, right_data;
+              get_data(db, left_path, left_id, left_from_db, left_data);
+              get_data(db, right_path, right_id, right_from_db, right_data);
+
+              string encloser("");
+              if (show_encloser)
+                lua.hook_get_encloser_pattern(right_path, encloser);
+
+              dump_diff(lua, 
+                        left_path, right_path,
+                        left_id, right_id,
+                        left_data, right_data,
+                        diff_format, external_diff_args_given, external_diff_args,
+                        encloser, output);
+            }
+          break;
+        }
+    }
 }
 
 // common functionality for diff and automate content_diff to determine
@@ -362,17 +237,16 @@ dump_diffs(lua_hooks & lua,
 static void
 prepare_diff(app_state & app,
              database & db,
-             cset & included,
+             roster_t & old_roster,
+             roster_t & new_roster,
              args_vector args,
-             bool & new_is_archived,
-             bool & old_is_archived,
+             bool & old_from_db,
+             bool & new_from_db,
              std::string & revheader)
 {
   temp_node_id_source nis;
   ostringstream header;
 
-  // The resulting diff is output in 'included'.
-
   // initialize before transaction so we have a database to work with.
   project_t project(db);
 
@@ -384,7 +258,7 @@ prepare_diff(app_state & app,
 
   if (app.opts.revision_selectors.empty())
     {
-      roster_t old_roster, restricted_roster, new_roster;
+      roster_t left_roster, restricted_roster, right_roster;
       revision_id old_rid;
       parent_map parents;
       workspace work(app);
@@ -397,71 +271,75 @@ prepare_diff(app_state & app,
           "(specify a revision to diff against with --revision)"));
 
       old_rid = parent_id(parents.begin());
-      old_roster = parent_roster(parents.begin());
-      work.get_current_roster_shape(db, nis, new_roster);
+      left_roster = parent_roster(parents.begin());
+      work.get_current_roster_shape(db, nis, right_roster);
 
       node_restriction mask(args_to_paths(args),
                             args_to_paths(app.opts.exclude_patterns),
                             app.opts.depth,
-                            old_roster, new_roster, ignored_file(work));
+                            left_roster, right_roster, ignored_file(work));
 
-      work.update_current_roster_from_filesystem(new_roster, mask);
+      work.update_current_roster_from_filesystem(right_roster, mask);
 
-      make_restricted_roster(old_roster, new_roster, restricted_roster,
+      make_restricted_roster(left_roster, right_roster, restricted_roster,
                              mask);
 
-      make_cset(old_roster, restricted_roster, included);
+      old_roster = left_roster;
+      new_roster = restricted_roster;
+
+      old_from_db = true;
+      new_from_db = false;
 
-      new_is_archived = false;
-      old_is_archived = true;
       header << "# old_revision [" << old_rid << "]\n";
     }
   else if (app.opts.revision_selectors.size() == 1)
     {
-      roster_t old_roster, restricted_roster, new_roster;
+      roster_t left_roster, restricted_roster, right_roster;
       revision_id r_old_id;
       workspace work(app);
 
       complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), r_old_id);
 
-      db.get_roster(r_old_id, old_roster);
-      work.get_current_roster_shape(db, nis, new_roster);
+      db.get_roster(r_old_id, left_roster);
+      work.get_current_roster_shape(db, nis, right_roster);
 
       node_restriction mask(args_to_paths(args),
                             args_to_paths(app.opts.exclude_patterns),
                             app.opts.depth,
-                            old_roster, new_roster, ignored_file(work));
+                            left_roster, right_roster, ignored_file(work));
 
-      work.update_current_roster_from_filesystem(new_roster, mask);
+      work.update_current_roster_from_filesystem(right_roster, mask);
 
-      make_restricted_roster(old_roster, new_roster, restricted_roster,
+      make_restricted_roster(left_roster, right_roster, restricted_roster,
                              mask);
 
       if (app.opts.reverse)
         {
-          make_cset(restricted_roster, old_roster, included);
-          new_is_archived = true;
-          old_is_archived = false;
+          old_roster = restricted_roster;
+          new_roster = left_roster;
+          old_from_db = false;
+          new_from_db = true;
         }
       else
         {
-          make_cset(old_roster, restricted_roster, included);
-          new_is_archived = false;
-          old_is_archived = true;
+          old_roster = left_roster;
+          new_roster = restricted_roster;
+          old_from_db = true;
+          new_from_db = false;
         }
 
       header << "# old_revision [" << r_old_id << "]\n";
     }
   else if (app.opts.revision_selectors.size() == 2)
     {
-      roster_t old_roster, restricted_roster, new_roster;
+      roster_t left_roster, restricted_roster, right_roster;
       revision_id r_old_id, r_new_id;
 
       complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), r_old_id);
       complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 1)(), r_new_id);
 
-      db.get_roster(r_old_id, old_roster);
-      db.get_roster(r_new_id, new_roster);
+      db.get_roster(r_old_id, left_roster);
+      db.get_roster(r_new_id, right_roster);
 
       // FIXME: this is *possibly* a UI bug, insofar as we
       // look at the restriction name(s) you provided on the command
@@ -488,30 +366,34 @@ prepare_diff(app_state & app,
       node_restriction mask(args_to_paths(args),
                             args_to_paths(app.opts.exclude_patterns),
                             app.opts.depth,
-                            old_roster, new_roster);
+                            left_roster, right_roster);
 
-      make_restricted_roster(old_roster, new_roster, restricted_roster,
+      make_restricted_roster(left_roster, right_roster, restricted_roster,
                              mask);
 
-      make_cset(old_roster, restricted_roster, included);
-
-      new_is_archived = true;
-      old_is_archived = true;
+      old_roster = left_roster;
+      new_roster = restricted_roster;
 
+      old_from_db = true;
+      new_from_db = true;
     }
   else
     {
       I(false);
     }
 
-    revheader = header.str();
+  revheader = header.str();
 }
 
 void dump_header(std::string const & revs,
-                 cset const & changes,
+                 roster_t const & old_roster,
+                 roster_t const & new_roster,
                  std::ostream & out,
                  bool show_if_empty)
 {
+  cset changes;
+  make_cset(old_roster, new_roster, changes);
+
   data summary;
   write_cset(changes, summary);
   if (summary().empty() && !show_if_empty)
@@ -542,37 +424,33 @@ CMD(diff, "diff", "di", CMD_REF(informative), N_("[PATH]..."),
        "that revision is shown.  If two revisions are given, the diff "
        "between them is given.  If no format is specified, unified is "
        "used by default."),
-    options::opts::revision | options::opts::depth | options::opts::exclude
-    | options::opts::diff_options)
+    options::opts::revision | options::opts::depth | options::opts::exclude |
+    options::opts::diff_options)
 {
   if (app.opts.external_diff_args_given)
     E(app.opts.diff_format == external_diff, origin::user,
       F("--diff-args requires --external\n"
         "try adding --external or removing --diff-args?"));
 
-  cset included;
+  roster_t old_roster, new_roster;
   std::string revs;
-  bool new_is_archived;
-  bool old_is_archived;
+  bool old_from_db;
+  bool new_from_db;
   database db(app);
 
-  prepare_diff(app, db, included, args, new_is_archived, old_is_archived, revs);
+  prepare_diff(app, db, old_roster, new_roster, args, old_from_db, new_from_db, revs);
 
   if (!app.opts.without_header)
     {
-      dump_header(revs, included, cout, true);
+      dump_header(revs, old_roster, new_roster, cout, true);
     }
 
-  if (app.opts.diff_format == external_diff)
-    {
-      do_external_diff(app.opts, app.lua, db, included, new_is_archived);
-    }
-  else
-    {
-      dump_diffs(app.lua, db, included, cout,
-                 app.opts.diff_format, new_is_archived, old_is_archived,
-                 !app.opts.no_show_encloser);
-    }
+  dump_diffs(app.lua, db, old_roster, new_roster, cout,
+             app.opts.diff_format,
+             app.opts.external_diff_args_given,
+             app.opts.external_diff_args,
+             old_from_db, new_from_db,
+             !app.opts.no_show_encloser);
 }
 
 
@@ -592,117 +470,56 @@ CMD_AUTOMATE(content_diff, N_("[FILE [...]]"),
              options::opts::revision | options::opts::depth |
              options::opts::exclude | options::opts::reverse)
 {
-  cset included;
-  std::string dummy_header;
-  bool new_is_archived;
-  bool old_is_archived;
+  roster_t old_roster, new_roster;
+  string dummy_header;
+  bool old_from_db;
+  bool new_from_db;
   database db(app);
 
-  prepare_diff(app, db, included, args, new_is_archived, old_is_archived, dummy_header);
+  prepare_diff(app, db, old_roster, new_roster, args, old_from_db, new_from_db,
+               dummy_header);
 
 
   if (app.opts.with_header)
     {
-      dump_header(dummy_header, included, output, false);
+      dump_header(dummy_header, old_roster, new_roster, output, false);
     }
 
-  dump_diffs(app.lua, db, included, output,
-             app.opts.diff_format, new_is_archived, old_is_archived, !app.opts.no_show_encloser);
+  dump_diffs(app.lua, db, old_roster, new_roster, output,
+             app.opts.diff_format,
+             app.opts.external_diff_args_given, app.opts.external_diff_args,
+             old_from_db, new_from_db, !app.opts.no_show_encloser);
 }
 
 
 static void
 log_certs(vector<cert> const & certs, ostream & os, cert_name const & name,
-          char const * label, char const * separator,
-          bool multiline, bool newline)
+          string const date_fmt = "")
 {
   bool first = true;
 
-  if (multiline)
-    newline = true;
-
-  for (vector<cert>::const_iterator i = certs.begin();
-       i != certs.end(); ++i)
+  for (vector<cert>::const_iterator i = certs.begin(); i != certs.end(); ++i)
     {
       if (i->name == name)
         {
           if (first)
-            os << label;
+            os << " ";
           else
-            os << separator;
+            os << ",";
 
-          if (multiline)
-            os << "\n\n";
-          os << i->value;
-          if (newline)
-            os << '\n';
-
-          first = false;
-        }
-    }
-}
-
-static void
-log_certs(vector<cert> const & certs, ostream & os, cert_name const & name,
-          char const * label, bool multiline)
-{
-  log_certs(certs, os, name, label, label, multiline, true);
-}
-
-static void
-log_certs(vector<cert> const & certs, ostream & os, cert_name const & name)
-{
-  log_certs(certs, os, name, " ", ",", false, false);
-}
-
-static void
-log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt,
-               char const * label, char const * separator,
-               bool multiline, bool newline)
-{
-  cert_name const date_name(date_cert_name);
-
-  bool first = true;
-  if (multiline)
-    newline = true;
-
-  for (vector<cert>::const_iterator i = certs.begin();
-       i != certs.end(); ++i)
-    {
-      if (i->name == date_name)
-        {
-          if (first)
-            os << label;
-          else
-            os << separator;
-
-          if (multiline)
-            os << "\n\n";
-          if (fmt.empty())
+          if (date_fmt.empty())
             os << i->value;
           else
-            os << date_t(i->value()).as_formatted_localtime(fmt);
-          if (newline)
-            os << '\n';
-
+            {
+              I(name == date_cert_name);
+              os << date_t(i->value()).as_formatted_localtime(date_fmt);
+            }
+            
           first = false;
         }
     }
 }
 
-static void
-log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt,
-               char const * label, bool multiline)
-{
-  log_date_certs(certs, os, fmt, label, label, multiline, true);
-}
-
-static void
-log_date_certs(vector<cert> const & certs, ostream & os, string const & fmt)
-{
-  log_date_certs(certs, os, fmt, " ", ",", false, false);
-}
-
 enum log_direction { log_forward, log_reverse };
 
 struct rev_cmp
@@ -748,7 +565,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       if (!app.opts.date_fmt.empty())
         date_fmt = app.opts.date_fmt;
       else
-        app.lua.hook_get_date_format_spec(date_time_short, date_fmt);
+        app.lua.hook_get_date_format_spec(date_time_long, date_fmt);
     }
 
   long last = app.opts.last;
@@ -781,9 +598,13 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       for (edge_map::const_iterator i = rev.edges.begin();
            i != rev.edges.end(); i++)
         {
-          starting_revs.insert(edge_old_revision(i));
+          revision_id rid = edge_old_revision(i);
+          E(db.revision_exists(rid), origin::user,
+            F("workspace parent revision '%s' not found - "
+              "did you specify a wrong database?") % rid);
+          starting_revs.insert(rid);
           if (i == rev.edges.begin())
-            first_rid = edge_old_revision(i);
+            first_rid = rid;
         }
     }
   else if (!app.opts.from.empty())
@@ -918,6 +739,7 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
     }
 
   cert_name const author_name(author_cert_name);
+  cert_name const date_name(date_cert_name);
   cert_name const branch_name(branch_cert_name);
   cert_name const tag_name(tag_cert_name);
   cert_name const changelog_name(changelog_cert_name);
@@ -935,7 +757,6 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
       revision_id const & rid = frontier.top().second;
 
       bool print_this = mask.empty();
-      set<file_path> diff_paths;
 
       if (null_id(rid) || seen.find(rid) != seen.end())
         {
@@ -990,12 +811,6 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
                   if (roster.has_node(*n) && mask.includes(roster, *n))
                     {
                       print_this = true;
-                      if (app.opts.diffs)
-                        {
-                          file_path fp;
-                          roster.get_name(*n, fp);
-                          diff_paths.insert(fp);
-                        }
                     }
                 }
             }
@@ -1036,58 +851,61 @@ CMD(log, "log", "", CMD_REF(informative), N_("[PATH] ..."),
               out << rid;
               log_certs(certs, out, author_name);
               if (app.opts.no_graph)
-                log_date_certs(certs, out, date_fmt);
+                log_certs(certs, out, date_name, date_fmt);
               else
                 {
                   out << '\n';
-                  log_date_certs(certs, out, date_fmt, "", "", false, false);
+                  log_certs(certs, out, date_name, date_fmt);
                 }
               log_certs(certs, out, branch_name);
               out << '\n';
             }
           else
             {
-              out << string(65, '-') << '\n';
-              out << _("Revision: ") << rid << '\n';
+              utf8 header;
+              revision_header(rid, rev, certs, date_fmt, header);
 
-              changes_summary csum;
+              external header_external;
+              utf8_to_system_best_effort(header, header_external);
+              out << header_external;
 
-              set<revision_id> ancestors;
-
-              for (edge_map::const_iterator e = rev.edges.begin();
-                   e != rev.edges.end(); ++e)
+              if (!app.opts.no_files)
                 {
-                  ancestors.insert(edge_old_revision(e));
-                  csum.add_change_set(edge_changes(e));
+                  utf8 summary;
+                  revision_summary(rev, summary);
+                  external summary_external;
+                  utf8_to_system_best_effort(summary, summary_external);
+                  out << summary_external;
                 }
-
-              for (set<revision_id>::const_iterator anc = ancestors.begin();
-                   anc != ancestors.end(); ++anc)
-                out << _("Ancestor: ") << *anc << '\n';
-
-              log_certs(certs, out, author_name,   _("Author: "), false);
-              log_date_certs(certs, out, date_fmt, _("Date: "), false);
-              log_certs(certs, out, branch_name,   _("Branch: "), false);
-              log_certs(certs, out, tag_name,      _("Tag: "),    false);
-
-              if (!app.opts.no_files && !csum.cs.empty())
-                {
-                  out << '\n';
-                  csum.print(out, 70);
-                  out << '\n';
-                }
-
-              log_certs(certs, out, changelog_name, _("ChangeLog: "), true);
-              log_certs(certs, out, comment_name,   _("Comments: "),  true);
             }
 
           if (app.opts.diffs)
             {
+              // if the current roster was loaded above this should hit the
+              // cache and not cost much... logging diffs isn't superfast
+              // regardless.
+              roster_t current_roster;
+              db.get_roster(rid, current_roster);
+
               for (edge_map::const_iterator e = rev.edges.begin();
                    e != rev.edges.end(); ++e)
-                dump_diffs(app.lua, db, edge_changes(e), diff_paths, out,
-                           app.opts.diff_format, true, true,
-                           !app.opts.no_show_encloser, !mask.empty());
+                {
+                  roster_t parent_roster, restricted_roster;
+
+                  db.get_roster(edge_old_revision(e), parent_roster);
+
+                  // always show forward diffs from the parent roster to
+                  // the current roster regardless of the log direction
+                  make_restricted_roster(parent_roster, current_roster,
+                                         restricted_roster, mask);
+
+                  dump_diffs(app.lua, db, parent_roster, restricted_roster,
+                             out, app.opts.diff_format, 
+                             app.opts.external_diff_args_given,
+                             app.opts.external_diff_args,
+                             true, true,
+                             !app.opts.no_show_encloser);
+                }
             }
 
           if (next > 0)
