@@ -20,6 +20,7 @@
 #include "project.hh"
 #include "keys.hh"
 #include "key_store.hh"
+#include "maybe_workspace_updater.hh"
 #include "transforms.hh"
 #include "vocab_cast.hh"
 
@@ -71,7 +72,9 @@ CMD(dropkey, "dropkey", "", CMD_REF(key_and_cert), N_("KEY_NAME_OR_HASH"),
 
   key_identity_info identity;
   project_t project(db);
-  project.get_key_identity(keys, app.lua, idx(args, 0), identity);
+  project.get_key_identity(keys, app.lua,
+                           typecast_vocab<external_key_name>(idx(args, 0)),
+                           identity);
 
   if (db.database_specified())
     {
@@ -116,7 +119,9 @@ CMD(passphrase, "passphrase", "", CMD_REF(key_and_cert), N_("KEY_NAME_OR_HASH"),
   project_t project(db);
   key_identity_info identity;
 
-  project.get_key_identity(keys, app.lua, idx(args, 0), identity);
+  project.get_key_identity(keys, app.lua,
+                           typecast_vocab<external_key_name>(idx(args, 0)),
+                           identity);
 
   keys.change_key_passphrase(identity.id);
   P(F("passphrase changed"));
@@ -232,7 +237,9 @@ CMD(trusted, "trusted", "", CMD_REF(key_and_cert),
   for (unsigned int i = 3; i != args.size(); ++i)
     {
       key_identity_info identity;
-      project.get_key_identity(keys, app.lua, idx(args, i), identity);
+      project.get_key_identity(keys, app.lua,
+                               typecast_vocab<external_key_name>(idx(args, i)),
+                               identity);
       signers.insert(identity);
     }
 
@@ -303,7 +310,7 @@ CMD(testresult, "testresult", "", CMD_REF(review),
 CMD(approve, "approve", "", CMD_REF(review), N_("REVISION"),
     N_("Approves a particular revision"),
     "",
-    options::opts::branch)
+    options::opts::branch | options::opts::maybe_auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -312,6 +319,8 @@ CMD(approve, "approve", "", CMD_REF(review), N_("REVISION"),
   if (args.size() != 1)
     throw usage(execid);
 
+  maybe_workspace_updater updater(app, project);
+
   revision_id r;
   complete(app.opts, app.lua, project, idx(args, 0)(), r);
   guess_branch(app.opts, project, r);
@@ -320,12 +329,14 @@ CMD(approve, "approve", "", CMD_REF(review), N_("REVISION"),
 
   cache_user_key(app.opts, app.lua, db, keys, project);
   project.put_revision_in_branch(keys, r, app.opts.branch);
+
+  updater.maybe_do_update();
 }
 
 CMD(suspend, "suspend", "", CMD_REF(review), N_("REVISION"),
     N_("Suspends a particular revision"),
     "",
-    options::opts::branch)
+    options::opts::branch | options::opts::maybe_auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -334,6 +345,8 @@ CMD(suspend, "suspend", "", CMD_REF(review), N_("REVISION"),
   if (args.size() != 1)
     throw usage(execid);
 
+  maybe_workspace_updater updater(app, project);
+
   revision_id r;
   complete(app.opts, app.lua, project, idx(args, 0)(), r);
   guess_branch(app.opts, project, r);
@@ -342,6 +355,8 @@ CMD(suspend, "suspend", "", CMD_REF(review), N_("REVISION"),
 
   cache_user_key(app.opts, app.lua, db, keys, project);
   project.suspend_revision_in_branch(keys, r, app.opts.branch);
+
+  updater.maybe_do_update();
 }
 
 CMD(comment, "comment", "", CMD_REF(review), N_("REVISION [COMMENT]"),
@@ -362,7 +377,7 @@ CMD(comment, "comment", "", CMD_REF(review), N_("REVISION [COMMENT]"),
   else
     {
       external comment_external;
-      E(app.lua.hook_edit_comment(external(""), external(""), comment_external),
+      E(app.lua.hook_edit_comment(external(""), comment_external),
         origin::user,
         F("edit comment failed"));
       system_to_utf8(comment_external, comment);
