@@ -1,5 +1,5 @@
 // Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
-//               2008 Stephen Leake <stephen_leake@stephe-leake.org>
+//               2008, 2010 Stephen Leake <stephen_leake@stephe-leake.org>
 //
 // This program is made available under the GNU GPL version 2.0 or
 // greater. See the accompanying file COPYING for details.
@@ -25,6 +25,7 @@
 #include "work.hh"
 #include "safe_map.hh"
 #include "app_state.hh"
+#include "maybe_workspace_updater.hh"
 #include "project.hh"
 #include "simplestring_xform.hh"
 #include "keys.hh"
@@ -175,21 +176,11 @@ pick_branch_for_update(options & opts, database & db,
   return switched_branch;
 }
 
-CMD(update, "update", "", CMD_REF(workspace), "",
-    N_("Updates the workspace"),
-    N_("This command modifies your workspace to be based off of a "
-       "different revision, preserving uncommitted changes as it does so.  "
-       "If a revision is given, update the workspace to that revision.  "
-       "If not, update the workspace to the head of the branch."),
-    options::opts::branch | options::opts::revision |
-    options::opts::move_conflicting_paths)
+// also used from maybe_workspace_updater.cc
+void
+update(app_state & app,
+       args_vector const & args)
 {
-  if (!args.empty())
-    throw usage(execid);
-
-  if (app.opts.revision_selectors.size() > 1)
-    throw usage(execid);
-
   database db(app);
   workspace work(app);
   project_t project(db);
@@ -252,7 +243,7 @@ CMD(update, "update", "", CMD_REF(workspace), "",
       P(F("already up to date at %s") % old_rid);
       // do still switch the workspace branch, in case they have used
       // update to switch branches.
-      work.set_options(app.opts, true);
+      work.set_options(app.opts, app.lua, true);
       return;
     }
 
@@ -321,7 +312,7 @@ CMD(update, "update", "", CMD_REF(workspace), "",
   content_merge_workspace_adaptor wca(db, old_rid, old_roster,
                                       left_markings, right_markings, paths);
   wca.cache_roster(working_rid, working_roster);
-  resolve_merge_conflicts(app.lua, *working_roster, chosen_roster,
+  resolve_merge_conflicts(app.lua, app.opts, *working_roster, chosen_roster,
                           result, wca, false);
 
   // Make sure it worked...
@@ -342,13 +333,46 @@ CMD(update, "update", "", CMD_REF(workspace), "",
   work.put_update_id(old_rid);
   work.put_work_rev(remaining);
   work.maybe_update_inodeprints(db);
-  work.set_options(app.opts, true);
+  work.set_options(app.opts, app.lua, true);
 
   if (switched_branch)
     P(F("switched branch; next commit will use branch %s") % app.opts.branch());
   P(F("updated to base revision %s") % chosen_rid);
 }
 
+CMD(update, "update", "", CMD_REF(workspace), "",
+    N_("Updates the workspace"),
+    N_("This command modifies your workspace to be based off of a "
+       "different revision, preserving uncommitted changes as it does so.  "
+       "If a revision is given, update the workspace to that revision.  "
+       "If not, update the workspace to the head of the branch."),
+    options::opts::branch | options::opts::revision |
+    options::opts::move_conflicting_paths)
+{
+  if (!args.empty())
+    throw usage(execid);
+
+  if (app.opts.revision_selectors.size() > 1)
+    throw usage(execid);
+
+  update(app, args);
+}
+
+CMD_AUTOMATE(update, "",
+             N_("Updates the workspace"),
+             "",
+             options::opts::branch | options::opts::revision |
+             options::opts::move_conflicting_paths)
+{
+  E(args.empty(), origin::user,
+    F("wrong argument count"));
+
+  E(app.opts.revision_selectors.size() <= 1, origin::user,
+    F("at most one revision selector may be specified"));
+
+  update(app, args);
+}
+
 // Subroutine of CMD(merge) and CMD(explicit_merge).  Merge LEFT with RIGHT,
 // placing results onto BRANCH.  Note that interactive_merge_and_store may
 // bomb out, and therefore so may this.
@@ -472,12 +496,15 @@ CMD(merge, "merge", "", CMD_REF(tree), "",
     N_("Merges unmerged heads of a branch"),
     "",
     options::opts::branch | options::opts::date | options::opts::author |
-    options::opts::messages | options::opts::resolve_conflicts_opts)
+    options::opts::messages | options::opts::resolve_conflicts_opts |
+    options::opts::maybe_auto_update)
 {
   database db(app);
   key_store keys(app);
   project_t project(db);
 
+  maybe_workspace_updater updater(app, project);
+
   if (!args.empty())
     throw usage(execid);
 
@@ -541,7 +568,7 @@ CMD(merge, "merge", "", CMD_REF(tree), "",
   if (heads.size() > 1)
     P(F("note: branch '%s' still has %d heads; run merge again") % app.opts.branch % heads.size());
 
-  P(F("note: your workspaces have not been updated"));
+  updater.maybe_do_update();
 }
 
 CMD(propagate, "propagate", "", CMD_REF(tree),
@@ -589,7 +616,7 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
     N_("Merges one branch into a subdirectory in another branch"),
     "",
     options::opts::date | options::opts::author | options::opts::messages |
-    options::opts::resolve_conflicts_opts)
+    options::opts::resolve_conflicts_opts | options::opts::maybe_auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -599,6 +626,8 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
   if (args.size() != 3)
     throw usage(execid);
 
+  maybe_workspace_updater updater(app, project);
+
   project.get_branch_heads(typecast_vocab<branch_name>(idx(args, 0)), src_heads,
                            app.opts.ignore_suspend_certs);
   project.get_branch_heads(typecast_vocab<branch_name>(idx(args, 1)), dst_heads,
@@ -699,7 +728,7 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
         parse_resolve_conflicts_opts
           (app.opts, left_rid, left_roster, right_rid, right_roster, result, resolutions_given);
 
-        resolve_merge_conflicts(app.lua, left_roster, right_roster,
+        resolve_merge_conflicts(app.lua, app.opts, left_roster, right_roster,
                                 result, dba, resolutions_given);
 
         {
@@ -734,6 +763,8 @@ CMD(merge_into_dir, "merge_into_dir", "", CMD_REF(tree),
       guard.commit();
       P(F("[merged] %s") % merged);
     }
+
+  updater.maybe_do_update();
 }
 
 CMD(merge_into_workspace, "merge_into_workspace", "", CMD_REF(tree),
@@ -821,7 +852,7 @@ CMD(merge_into_workspace, "merge_into_workspace", "", CMD_REF(tree),
   content_merge_workspace_adaptor wca(db, lca_id, lca.first,
                                       *left.second, *right.second, paths);
   wca.cache_roster(working_rid, working_roster);
-  resolve_merge_conflicts(app.lua, *left.first, *right.first, merge_result, wca, false);
+  resolve_merge_conflicts(app.lua, app.opts, *left.first, *right.first, merge_result, wca, false);
 
   // Make sure it worked...
   I(merge_result.is_clean());
@@ -859,7 +890,8 @@ CMD(explicit_merge, "explicit_merge", "", CMD_REF(tree),
     N_("The results of the merge are placed on the branch specified by "
        "DEST-BRANCH."),
     options::opts::date | options::opts::author |
-    options::opts::messages | options::opts::resolve_conflicts_opts)
+    options::opts::messages | options::opts::resolve_conflicts_opts |
+    options::opts::maybe_auto_update)
 {
   database db(app);
   key_store keys(app);
@@ -870,6 +902,8 @@ CMD(explicit_merge, "explicit_merge", "", CMD_REF(tree),
   if (args.size() != 3)
     throw usage(execid);
 
+  maybe_workspace_updater updater(app, project);
+
   complete(app.opts, app.lua, project, idx(args, 0)(), left);
   complete(app.opts, app.lua, project, idx(args, 1)(), right);
   branch = typecast_vocab<branch_name>(idx(args, 2));
@@ -889,6 +923,8 @@ CMD(explicit_merge, "explicit_merge", "", CMD_REF(tree),
   merge_two(app.opts, app.lua, project, keys,
             left, right, branch, string("explicit merge"),
             std::cout, false);
+
+  updater.maybe_do_update();
 }
 
 namespace
@@ -1325,7 +1361,7 @@ CMD(pluck, "pluck", "", CMD_REF(workspace), N_("[-r FROM] -r TO [PATH...]"),
   // to_roster is not fetched from the db which does not have temporary nids
   wca.cache_roster(to_rid, to_roster);
 
-  resolve_merge_conflicts(app.lua, *working_roster, *to_roster,
+  resolve_merge_conflicts(app.lua, app.opts, *working_roster, *to_roster,
                           result, wca, false);
 
   I(result.is_clean());
