@@ -37,7 +37,9 @@ CMD(pubkey, "pubkey", "", CMD_REF(packet_io), N_("ID"),
   project_t project(db);
 
   key_identity_info identity;
-  project.get_key_identity(keys, app.lua, idx(args, 0), identity);
+  project.get_key_identity(keys, app.lua,
+                           typecast_vocab<external_key_name>(idx(args, 0)),
+                           identity);
   bool exists(false);
   rsa_pub_key key;
   if (db.database_specified() && db.public_key_exists(identity.id))
@@ -73,7 +75,9 @@ CMD(privkey, "privkey", "", CMD_REF(packet_io), N_("ID"),
 
   key_name name = typecast_vocab<key_name>(idx(args, 0));
   key_identity_info identity;
-  project.get_key_identity(app.lua, idx(args, 0), identity);
+  project.get_key_identity(app.lua,
+                           typecast_vocab<external_key_name>(idx(args, 0)),
+                           identity);
   E(keys.key_pair_exists(identity.id), origin::user,
     F("public and private key '%s' do not exist in keystore")
     % idx(args, 0)());
