@@ -111,7 +111,8 @@ file_exists(any_path const & p)
   return get_path_status(p) == path::file;
 }
 
-namespace
+bool
+directory_empty(any_path const & path)
 {
   struct directory_not_empty_exception {};
   struct directory_empty_helper : public dirent_consumer
@@ -119,53 +120,47 @@ namespace
     virtual void consume(char const *)
     { throw directory_not_empty_exception(); }
   };
-}
 
-bool
-directory_empty(any_path const & path)
-{
   directory_empty_helper h;
   try {
-    do_read_directory(system_path(path).as_external(), h, h, h);
+    read_directory(path, h, h, h);
   } catch (directory_not_empty_exception) {
     return false;
   }
   return true;
 }
 
-static bool did_char_is_binary_init;
-static bool char_is_binary[256];
-
-static void
-set_char_is_binary(char c, bool is_binary)
-{
-    char_is_binary[static_cast<u8>(c)] = is_binary;
-}
-
-static void
-init_char_is_binary()
-{
-  // these do not occur in ASCII text files
-  // FIXME: this heuristic is (a) crap and (b) hardcoded. fix both these.
-  // Should be calling a lua hook here that can use set_char_is_binary()
-  // That will at least fix (b)
-  string nontext_chars("\x01\x02\x03\x04\x05\x06\x0e\x0f"
-                       "\x10\x11\x12\x13\x14\x15\x16\x17\x18"
-                       "\x19\x1a\x1c\x1d\x1e\x1f");
-  set_char_is_binary('\0', true);
-  for(size_t i = 0; i < nontext_chars.size(); ++i)
-    {
-      set_char_is_binary(nontext_chars[i], true);
-    }
-}
+// This is not the greatest heuristic ever; it just looks for characters in
+// the original ASCII control code range (00 - 1f, 7f) that are not white
+// space (07 - 0D).  But then, GNU diff just looks for NULs.  We could do
+// better if this was detecting character encoding (because then we could
+// report wide encodings as such instead of treating them as binary) but the
+// application proper isn't set up for that.
+//
+// Everything > 128 *can* be a valid text character, depending on encoding,
+// even in the 80 - 9F region that Unicode reserves for yet more useless
+// control characters.
+//
+// N.B. the obvious algorithmic version of the inner loop here
+//     u8 c = s[i];
+//     if (c <= 0x06 || (c >= 0x0E && c < 0x20) || c == 0x7F)
+//       return true;
+// is about twice as slow on current hardware (Intel Core2 quad).
 
 bool guess_binary(string const & s)
 {
-  if (did_char_is_binary_init == false)
-    {
-      init_char_is_binary();
-      did_char_is_binary_init = true;
-    }
+  static const bool char_is_binary[256] = {
+  //_0 _1 _2 _3 _4 _5 _6 _7 _8 _9 _A _B _C _D _E _F
+    1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, // 0_
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 1_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 2_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 3_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 4_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 5_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 6_
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, // 7_
+    0                                               // 8+
+  };
 
   for (size_t i = 0; i < s.size(); ++i)
     {
@@ -228,68 +223,6 @@ delete_file_or_dir_shallow(any_path const & p)
   do_remove(p.as_external());
 }
 
-namespace
-{
-  struct fill_pc_vec : public dirent_consumer
-  {
-    fill_pc_vec(vector<path_component> & v) : v(v) { v.clear(); }
-
-    // FIXME BUG: this treats 's' as being already utf8,
-    // but it is actually in the external character set.
-    virtual void consume(char const * s)
-    {
-      try
-      {
-        v.push_back(path_component(s));
-      }
-      catch (...)
-      {
-        W(F("skipping invalid path '%s'") % s);
-      }
-    }
-
-  private:
-    vector<path_component> & v;
-  };
-
-  struct file_deleter : public dirent_consumer
-  {
-    file_deleter(any_path const & p) : parent(p) {}
-    virtual void consume(char const * f)
-    {
-      try
-      {
-        do_remove((parent / path_component(f)).as_external());
-      }
-      catch (...)
-      {
-        W(F("skipping invalid path '%s'") % f);
-      }
-    }
-  private:
-    any_path const & parent;
-  };
-}
-
-static void
-do_remove_recursive(any_path const & p)
-{
-  // for the reasons described in walk_tree_recursive, we read the entire
-  // directory before recursing into any subdirs.  however, it is safe to
-  // delete files as we encounter them, and we do so.
-  vector<path_component> subdirs;
-  fill_pc_vec subdir_fill(subdirs);
-  file_deleter delete_files(p);
-
-  do_read_directory(p.as_external(), delete_files, subdir_fill, delete_files);
-  for (vector<path_component>::const_iterator i = subdirs.begin();
-       i != subdirs.end(); i++)
-    do_remove_recursive(p / *i);
-
-  do_remove(p.as_external());
-}
-
-
 void
 delete_dir_recursive(any_path const & p)
 {
@@ -297,7 +230,7 @@ delete_dir_recursive(any_path const & p)
                             F("directory to delete, '%s', does not exist") % p,
                             F("directory to delete, '%s', is a file") % p);
 
-  do_remove_recursive(p);
+  do_remove_recursive(p.as_external());
 }
 
 void
@@ -361,17 +294,6 @@ read_data(any_path const & p, data & dat)
              data_from);
 }
 
-void read_directory(any_path const & path,
-                    vector<path_component> & files,
-                    vector<path_component> & dirs)
-{
-  vector<path_component> special_files;
-  fill_pc_vec ff(files), df(dirs), sf(special_files);
-  do_read_directory(path.as_external(), ff, df, sf);
-  E(special_files.empty(), origin::system,
-    F("cannot handle special files in dir '%s'") % path);
-}
-
 // This function can only be called once per run.
 void
 read_data_stdin(data & dat)
@@ -458,40 +380,6 @@ tree_walker::visit_dir(file_path const & path)
   return true;
 }
 
-// subroutine of walk_tree_recursive: if the path composition of PATH and PC
-// is a valid file_path, write it to ENTRY and return true.  otherwise,
-// generate an appropriate diagnostic and return false.  in this context, an
-// invalid path is *not* an invariant failure, because it came from a
-// directory scan.  ??? arguably belongs as a file_path method.
-static bool
-safe_compose(file_path const & path, path_component const & pc, bool isdir,
-             file_path & entry)
-{
-  try
-    {
-      entry = path / pc;
-      return true;
-    }
-  catch (logic_error)
-    {
-      // do what the above operator/ did, by hand, and then figure out what
-      // sort of diagnostic to issue.
-      utf8 badpth;
-      if (path.empty())
-        badpth = typecast_vocab<utf8>(pc);
-      else
-        badpth = utf8(path.as_internal() + "/" + pc(), pc.made_from);
-
-      if (!isdir)
-        W(F("skipping file '%s' with unsupported name") % badpth);
-      else if (bookkeeping_path::internal_string_is_bookkeeping_path(badpth))
-        L(FL("ignoring bookkeeping directory '%s'") % badpth);
-      else
-        W(F("skipping directory '%s' with unsupported name") % badpth);
-      return false;
-    }
-}
-
 static void
 walk_tree_recursive(file_path const & path,
                     tree_walker & walker)
@@ -506,25 +394,20 @@ walk_tree_recursive(file_path const & path,
   // peak memory.  By splitting the loop in half, we avoid this problem.
   //
   // [1] http://lkml.org/lkml/2006/2/24/215
-  vector<path_component> files, dirs;
-  read_directory(path, files, dirs);
+  vector<file_path> files, dirs;
+  fill_path_vec<file_path> fill_files(path, files, false);
+  fill_path_vec<file_path> fill_dirs(path, dirs, true);
 
-  for (vector<path_component>::const_iterator i = files.begin();
+  read_directory(path, fill_files, fill_dirs);
+
+  for (vector<file_path>::const_iterator i = files.begin();
        i != files.end(); ++i)
-    {
-      file_path entry;
-      if (safe_compose(path, *i, false, entry))
-        walker.visit_file(entry);
-    }
+    walker.visit_file(*i);
 
-  for (vector<path_component>::const_iterator i = dirs.begin();
+  for (vector<file_path>::const_iterator i = dirs.begin();
        i != dirs.end(); ++i)
-    {
-      file_path entry;
-      if (safe_compose(path, *i, true, entry))
-        if (walker.visit_dir(entry))
-          walk_tree_recursive(entry, walker);
-    }
+    if (walker.visit_dir(*i))
+      walk_tree_recursive(*i, walker);
 }
 
 // from some (safe) sub-entry of cwd
