@@ -1,3 +1,12 @@
+// Copyright (C) 2006 Timothy Brownawell <tbrownaw@gmail.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
+
 #include "base.hh"
 #include "mtn-sanity.hh"
 #include "ui.hh"
@@ -26,7 +35,10 @@ mtn_sanity::initialize(int argc, char ** argv, char const * lc_all)
 void
 mtn_sanity::inform_log(std::string const &msg)
 {
-  ui.inform(msg);
+  if (debug_p())
+    {
+      ui.inform(msg);
+    }
 }
 
 void
@@ -47,7 +59,6 @@ mtn_sanity::inform_error(std::string const &msg)
   ui.inform(msg);
 }
 
-
 // Local Variables:
 // mode: C++
 // fill-column: 76
@@ -55,4 +66,3 @@ mtn_sanity::inform_error(std::string const &msg)
 // indent-tabs-mode: nil
 // End:
 // vim: et:sw=2:sts=2:ts=2:cino=>2s,{s,\:s,+s,t0,g0,^-2,e-2,n-2,p2s,(0,=s:
-
