@@ -1371,7 +1371,7 @@ cluster_consumer::store_auxiliary_certs(prepared_revision const & p)
                              branch_name(branchname, origin::user),
                              utf8(cvs.changelog_interner.lookup(p.changelog),
                                   origin::internal),
-                             date_t(p.time),
+                             date_t(s64(p.time) * 1000),
                              cvs.author_interner.lookup(p.author));
 }
 
