@@ -12,7 +12,8 @@ check(mtn("add", "input.txt"), 0, false, false)
 
 check(mtn("--branch=testbranch", "--rcfile=commit_log.lua", "commit"), 0, false, false)
 
-testURI="file:" .. test.root .. "/test.db"
+copy("test.db", "test-clone.db")
+testURI="file:" .. test.root .. "/test-clone.db"
 
 check(nodb_mtn("clone", testURI, "testbranch", "testbranch"), 0, false, true)
 
