@@ -5,7 +5,8 @@ mtn_setup()
 addfile("testfile", "foo")
 commit()
 
-testURI="file:" .. test.root .. "/test.db"
+copy("test.db", "test-clone.db")
+testURI="file:" .. test.root .. "/test-clone.db"
 
 check(nodb_mtn("clone", testURI, "testbranch", "test_dir1"), 0, false, false)
 
