@@ -5,9 +5,17 @@ function execute(path,...)
       return 1
    end
    if path == "editor" then
-      tmp = io.open(tname, "w")
-      tmp:write("Hello\n")
+
+      tmp = io.open(tname, "r")
+      text = tmp:read("*a")
       io.close(tmp)
+
+      text = string.gsub(text, "\nChangelog: \n\n\n", "\nChangelog: \n\nHello\n")
+
+      tmp = io.open(tname, "w")
+      tmp:write(text)
+      io.close(tmp)
+
       return 0
    end
    return 1
