@@ -32,12 +32,12 @@ check(get("alice.lua"))
 function bob (...)
   return raw_mtn("--rcfile=test_hooks.lua", "--rcfile=bob.lua",
                  "--nostd", "--norc", "--db=test.db", "--key=bob",
-                 "--keydir=keys", unpack(arg))
+                 "--keydir=keys", ...)
 end
 function alice (...)
   return raw_mtn("--rcfile=test_hooks.lua", "--rcfile=alice.lua",
                  "--nostd", "--norc", "--db=test.db", "--key=alice",
-                 "--keydir=keys", unpack(arg))
+                 "--keydir=keys", ...)
 end
 
 check(bob("genkey", "bob"), 0, false, false, "bob\nbob\n")
