@@ -14,7 +14,7 @@ copy("v1b", "testfile")
 commit()
 
 remove("testfile")
-check(mtn("drop", "testfile"), 0, false, false)
+check(mtn("drop", "--bookkeep-only", "testfile"), 0, false, false)
 commit()
 
 copy("v2a", "testfile")
