@@ -7,5 +7,6 @@ copy("test.db", "test2.db")
 addfile("testfile", "foo")
 commit()
 
-check(mtn("sync", "file:test2.db", "testbranch"), 0, false, false)
+check(mtn("sync", "file:test2.db", "testbranch"), 0, false, true)
+check(not qgrep("error", "stderr"))
 check_same_db_contents("test.db", "test2.db")
