@@ -17,5 +17,5 @@ selmap("p:" .. REV2, {REV1})
 -- single-character prefix
 selmap("p:" .. string.sub(REV2,1,1), {REV1})
 -- parent of a root revision
-selmap("p:" .. REV1, {})
+selmap("p:" .. REV1, {""})
 
