@@ -5,13 +5,10 @@ mkdir("dir")
 check(mtn("add", "dir"), 0, false, false)
 commit()
 
-check(mtn("automate", "get_revision"), 0, true, false)
-rename("stdout", "orig_rev")
-
 -- fail to move a dir under a file
 check(mtn("rename", "--bookkeep-only", "dir", "file/subdir"), 1, false, false)
-check(mtn("automate", "get_revision"), 0, true, false)
-check(samefile("stdout", "orig_rev"))
+check(mtn("status"), 0, true, false)
+check(qgrep("no changes", "stdout"))
 
 -- running a recursive add what's supposed to be a file, but is actually a
 -- dir...
