@@ -66,13 +66,14 @@ struct user_interface::impl
 };
 
 ticker::ticker(string const & tickname, string const & s, size_t mod,
-    bool kilocount) :
+    bool kilocount, bool skip_display) :
   ticks(0),
   mod(mod),
   total(0),
   previous_total(0),
   kilocount(kilocount),
   use_total(false),
+  may_skip_display(skip_display),
   keyname(tickname),
   name(_(tickname.c_str())),
   shortname(s),
@@ -154,6 +155,17 @@ private:
   unsigned int chars_on_line;
 };
 
+struct tick_write_stdio : virtual public tick_writer
+{
+public:
+  tick_write_stdio();
+  ~tick_write_stdio();
+  void write_ticks();
+  void clear_line();
+private:
+  std::map<std::string,size_t> last_ticks;
+};
+
 struct tick_write_nothing : virtual public tick_writer
 {
 public:
@@ -236,6 +248,12 @@ void tick_write_count::write_ticks()
     {
       ticker * tick = i->second;
 
+      // if the display of this ticker has no great importance, i.e. multiple
+      // other tickers should be displayed at the same time, skip its display
+      // to save space on terminals
+      if (tick->may_skip_display)
+        continue;
+
       if ((tick->count_size == 0 && tick->kilocount)
           ||
           (tick->use_total && tick->previous_total != tick->total))
@@ -428,6 +446,69 @@ void tick_write_dot::clear_line()
   clog << endl;
 }
 
+
+tick_write_stdio::tick_write_stdio()
+{}
+
+tick_write_stdio::~tick_write_stdio()
+{}
+
+void tick_write_stdio::write_ticks()
+{
+  I(ui.imp);
+  string headers, sizes, tickline;
+
+  for (map<string,ticker *>::const_iterator i = ui.imp->tickers.begin();
+       i != ui.imp->tickers.end(); ++i)
+    {
+      std::map<std::string, size_t>::iterator it =
+            last_ticks.find(i->second->shortname);
+
+      // we output each explanation stanza just once and every time the
+      // total count has been changed
+      if (it == last_ticks.end())
+        {
+          headers += i->second->shortname + ":" + i->second->name + ";";
+          sizes   += i->second->shortname + "=" +  lexical_cast<string>(i->second->total) + ";";
+          last_ticks[i->second->shortname] = i->second->total;
+        }
+      else
+      if (it->second != i->second->total)
+        {
+          sizes   += i->second->shortname + "=" +  lexical_cast<string>(i->second->total) + ";";
+          last_ticks[i->second->shortname] = i->second->total;
+        }
+
+      tickline += i->second->shortname + "#" + lexical_cast<string>(i->second->ticks) + ";";
+    }
+
+  if (!headers.empty())
+    {
+      global_sanity.maybe_write_to_out_of_band_handler('t', headers);
+    }
+  if (!sizes.empty())
+    {
+      global_sanity.maybe_write_to_out_of_band_handler('t', sizes);
+    }
+
+  I(!tickline.empty());
+  global_sanity.maybe_write_to_out_of_band_handler('t', tickline);
+}
+
+void tick_write_stdio::clear_line()
+{
+  std::map<std::string, size_t>::iterator it;
+  std::string out;
+
+  for (it = last_ticks.begin(); it != last_ticks.end(); it++)
+  {
+    out += it->first + ";";
+  }
+
+  global_sanity.maybe_write_to_out_of_band_handler('t', out);
+  last_ticks.clear();
+}
+
 // user_interface has both constructor/destructor and initialize/
 // deinitialize because there's only one of these objects, it's
 // global, and we don't want global constructors/destructors doing
@@ -446,6 +527,8 @@ void user_interface::initialize()
     set_tick_write_count();
   else
     set_tick_write_dot();
+
+  timestamps_enabled = false;
 }
 
 void user_interface::deinitialize()
@@ -478,27 +561,48 @@ void
 user_interface::set_tick_write_dot()
 {
   I(imp);
+  if (tick_type == dot)
+    return;
   if (imp->t_writer != 0)
     delete imp->t_writer;
   imp->t_writer = new tick_write_dot;
+  tick_type = dot;
 }
 
 void
 user_interface::set_tick_write_count()
 {
   I(imp);
+  if (tick_type == count)
+    return;
   if (imp->t_writer != 0)
     delete imp->t_writer;
   imp->t_writer = new tick_write_count;
+  tick_type = count;
+}
+
+void
+user_interface::set_tick_write_stdio()
+{
+  I(imp);
+  if (tick_type == stdio)
+    return;
+  if (imp->t_writer != 0)
+    delete imp->t_writer;
+  imp->t_writer = new tick_write_stdio;
+  tick_type = stdio;
 }
 
 void
 user_interface::set_tick_write_nothing()
 {
   I(imp);
+  if (tick_type == none)
+    return;
   if (imp->t_writer != 0)
     delete imp->t_writer;
   imp->t_writer = new tick_write_nothing;
+  tick_type = none;
 }
 
 
@@ -637,10 +741,31 @@ user_interface::fatal_exception()
 string
 user_interface::output_prefix()
 {
-  if (prog_name.empty()) {
-    return "?: ";
+  std::string prefix;
+
+  if (timestamps_enabled) {
+    try {
+      // FIXME: with no app pointer around we have no access to
+      // app.lua.get_date_format_spec() here, so we use the same format
+      // which f.e. also Apache uses for its log output
+      prefix = "[" +
+        date_t::now().as_formatted_localtime("%a %b %d %H:%M:%S %Y") +
+        "] ";
+    }
+    // ensure that we do not throw an exception because we could not
+    // create the timestamp prefix above
+    catch (...) {}
   }
-  return prog_name + ": ";
+
+  if (prog_name.empty()) {
+    prefix += "?: ";
+  }
+  else
+  {
+    prefix += prog_name + ": ";
+  }
+
+  return prefix;
 }
 
 static inline string
@@ -888,6 +1013,12 @@ user_interface::inform_usage(usage const & u, options & opts)
   commands::explain_usage(u.which, opts.show_hidden_commands, usage_stream);
 }
 
+void
+user_interface::enable_timestamps()
+{
+  timestamps_enabled = true;
+}
+
 // Local Variables:
 // mode: C++
 // fill-column: 76
