@@ -205,10 +205,10 @@ namespace
 }
 
 void
-do_read_directory(std::string const & path,
-                  dirent_consumer & files,
-                  dirent_consumer & dirs,
-                  dirent_consumer & /* specials */)
+read_directory(std::string const & path,
+               dirent_consumer & files,
+               dirent_consumer & dirs,
+               dirent_consumer & /* specials */)
 {
   dirhandle dir(path);
   WIN32_FIND_DATA d;
@@ -248,6 +248,54 @@ do_remove(std::string const & path)
     F("could not remove '%s': %s") % path % os_strerror(GetLastError()));
 }
 
+void
+do_remove_recursive(std::string const & path)
+{
+  // IMPROVME: use Unicode APIs to support more pathnames.
+  //
+  // SHFileOperation makes the weird requirement that its pFrom (and pTo)
+  // arguments be terminated with *two* ASCII NULs.
+  size_t pfLen = path.size();
+  LPSTR pFrom = (LPSTR)malloc(pfLen + 2);
+  memcpy(pFrom, path.data(), pfLen);
+  pFrom[pfLen] = '\0';
+  pFrom[pfLen+1] = '\0';
+
+  // SHFileOperationA can't handle '/'
+  for (size_t i = 0; i < pfLen; i++)
+    {
+      if (pFrom[i] == '/')
+          pFrom[i] = '\\';
+    };
+
+  SHFILEOPSTRUCTA op;
+  op.hwnd   = (HWND)INVALID_HANDLE_VALUE;
+  op.wFunc  = FO_DELETE;
+  op.pFrom  = pFrom;
+  op.pTo    = NULL;
+  op.fFlags = (FOF_SILENT | FOF_NOCONFIRMATION | FOF_NOCONFIRMMKDIR
+               | FOF_NOERRORUI); // aka FOF_NO_UI, but that's vista only
+  op.fAnyOperationsAborted = FALSE;
+  op.hNameMappings = INVALID_HANDLE_VALUE;
+  op.lpszProgressTitle = NULL;
+
+  int rc = SHFileOperationA(&op);
+  // http://msdn.microsoft.com/en-us/library/bb762164(VS.85).aspx
+  // warns that the return codes from SHFileOperation are *not* normal
+  // Win32 error codes; so we don't try to do os_strerror on them.
+  //
+  // 0x402 is "unknown error"; it occurs for a non-existing path, which is
+  // not an error in this function. It also occurs for other problems, like
+  // '/' as a directory separator. Sigh.
+  E(rc == 0 || rc == 0x402, origin::system,
+    F("could not remove '%s' and contents: SHFileOperation error code 0x%x")
+    % path % rc);
+  E(!op.fAnyOperationsAborted, origin::system,
+    F("could not remove '%s' and contents: SHFileOperation partially aborted")
+    % path);
+
+}
+
 void
 do_mkdir(std::string const & path)
 {
@@ -304,6 +352,19 @@ rename_clobberingly_impl(const char * from, const char * to)
   return MoveFile(from, to);
 }
 
+void make_accessible(std::string const & name)
+{
+  DWORD attrs = GetFileAttributes(name.c_str());
+  E(attrs != INVALID_FILE_ATTRIBUTES, origin::system,
+    F("GetFileAttributes(%s) failed: %s")
+    % name % os_strerror(GetLastError()));
+
+  E(SetFileAttributes(name.c_str(), attrs & ~FILE_ATTRIBUTE_READONLY),
+    origin::system,
+    F("SetFileAttributes(%s) failed: %s")
+    % name % os_strerror(GetLastError()));
+}
+
 void
 rename_clobberingly(std::string const & from, std::string const & to)
 {
