@@ -22,13 +22,19 @@ bool have_smart_terminal()
   else
     term = "";
 
-  // Win32 consoles are weird; cmd.exe does not set TERM, but isatty returns
-  // true, Cygwin and MinGW MSYS shells set a TERM but isatty returns false.
-  // Let's just check for some obvious dumb terminals, and default to smart.
-  if (term == "" || term == "dumb")
-    return false;
+  // cmd.exe does not set TERM, but isatty returns true. Cygwin and MinGW
+  // MSYS shells set a TERM but isatty returns false.
+  if (isatty(2))
+    {
+      return true;
+    }
   else
-    return true;
+    {
+      if (term == "" || term == "dumb")
+        return false;
+      else
+        return true;
+    }
 }
 
 unsigned int terminal_width()
