@@ -19,16 +19,6 @@
 
 #include <windows.h>
 
-void make_accessible(std::string const & name)
-{
-  DWORD attrs = GetFileAttributes(name.c_str());
-  E(attrs != INVALID_FILE_ATTRIBUTES, origin::system,
-    F("GetFileAttributes(%s) failed: %s") % name % os_strerror(GetLastError()));
-
-  E(SetFileAttributes(name.c_str(), attrs & ~FILE_ATTRIBUTE_READONLY), origin::system,
-    F("SetFileAttributes(%s) failed: %s") % name % os_strerror(GetLastError()));
-}
-
 time_t get_last_write_time(char const * name)
 {
   HANDLE h = CreateFile(name, GENERIC_READ, FILE_SHARE_READ, NULL,
@@ -212,7 +202,18 @@ void run_tests_in_children(test_enumerator const & next_test,
 
       DWORD end_millis = GetTickCount();
       if (cleanup(t, status, (end_millis - start_millis) / 1000, -1))
-        do_remove_recursive(testdir);
+        try
+          {
+            do_remove_recursive(testdir);
+          }
+        catch (...)
+          {
+            // process_wait sometimes returns before releasing the lock on
+            // the directory that we tried to remove. So wait a little
+            // longer and try again.
+            Sleep (1000);// milliseconds
+            do_remove_recursive(testdir);
+          }
     }
 }
 
